@@ -3,6 +3,8 @@ import time
 from typing import Any, Dict, List
 
 import httpx
+from django.contrib.auth import get_user_model
+from django.http import HttpRequest
 from django.urls import resolve
 
 from . import settings as app_settings
@@ -15,7 +17,6 @@ except ImportError:
 else:  # pragma: no cover
     CAN_GEOIP = True
 
-
 log = logging.getLogger(__name__)
 
 
@@ -73,34 +74,35 @@ class Amplitude():
 
         return event
 
-    def page_view_event(self, request):
+    def build_event_data(
+        self, event_type: str, request: HttpRequest, **kwargs
+    ) -> dict:
         """
-        Send an event based on a Django request
+        Build event data using a Django request object
         """
         event: Dict[str, Any] = {
             'device_id': request.session.get('amplitude_device_id'),
-            'event_type': f'Page view',
+            'event_type': event_type,
             'time': int(round(time.time() * 1000)),
             'ip': get_client_ip(request),
             'language': getattr(request, 'LANGUAGE_CODE', ''),
-            # 'app_version': '',
-            # 'device_id': '',
-            # 'carrier': '',
-            # 'dma': 'San Francisco-Oakland-San Jose, CA',
-            # 'price': 4.99,
-            # 'quantity': 3,
-            # 'revenue': -1.99,
-            # 'productId': 'Google Pay Store Product Id',
-            # 'revenueType': 'Refund',
-            # 'idfa': '',
-            # 'idfv': '',
-            # 'adid': '',
-            # 'android_id': '',
-            # 'event_id': 23,
-            # 'insert_id': '',
+            'app_version': kwargs.get('app_version'),
+            'carrier': kwargs.get('carrier'),
+            'dma': kwargs.get('dma'),
+            'price': kwargs.get('price'),
+            'quantity': kwargs.get('quantity'),
+            'revenue': kwargs.get('revenue'),
+            'productId': kwargs.get('productId'),
+            'revenueType': kwargs.get('revenueType'),
+            'idfa': kwargs.get('idfa'),
+            'idfv': kwargs.get('idfv'),
+            'adid': kwargs.get('adid'),
+            'android_id': kwargs.get('android_id'),
+            'event_id': kwargs.get('event_id'),
+            'insert_id': kwargs.get('insert_id'),
         }
         try:
-            event['user_id'] = f'{request.user.id:05}'
+            event['user_id'] = f'{request.user.pk:05}'
         except (AttributeError, TypeError):
             pass
 
@@ -112,9 +114,9 @@ class Amplitude():
         event.update(device_data)
         location_data = self.location_data_from_ip_address(event['ip'])
         event.update(location_data)
-        self.send_events(events=[event])
+        return event
 
-    def event_properties_from_request(self, request) -> dict:
+    def event_properties_from_request(self, request: HttpRequest) -> dict:
         url_name = resolve(request.path_info).url_name
         event_properties = {
             'url': request.path,
@@ -126,7 +128,7 @@ class Amplitude():
             event_properties['kwargs'] = request.resolver_match.kwargs
         return event_properties
 
-    def user_properties_from_request(self, request) -> dict:
+    def user_properties_from_request(self, request: HttpRequest) -> dict:
         try:
             request.user.is_authenticated
         except AttributeError:
@@ -135,17 +137,20 @@ class Amplitude():
         if not self.include_user_data or not request.user.is_authenticated:
             return {}
 
+        User = get_user_model()
+        user = User.objects.get(pk=request.user.pk)
+
         return {
-            'username': request.user.get_username(),
-            'email': request.user.email,
-            'full_name': request.user.get_full_name(),
-            'is_staff': request.user.is_staff,
-            'is_superuser': request.user.is_superuser,
-            'last_login': request.user.last_login.isoformat(),
-            'date_joined': request.user.date_joined.isoformat(),
+            'username': user.get_username(),
+            'email': user.email,
+            'full_name': user.get_full_name(),
+            'is_staff': user.is_staff,
+            'is_superuser': user.is_superuser,
+            'last_login': user.last_login.isoformat(),
+            'date_joined': user.date_joined.isoformat(),
         }
 
-    def group_from_request(self, request) -> list:
+    def group_from_request(self, request: HttpRequest) -> list:
         try:
             request.user.is_authenticated
         except AttributeError:
@@ -154,7 +159,9 @@ class Amplitude():
         if not self.include_group_data or not request.user.is_authenticated:
             return []
 
-        groups = request.user.groups.all().values_list('name', flat=True)
+        User = get_user_model()
+        user = User.objects.get(pk=request.user.pk)
+        groups = user.groups.all().values_list('name', flat=True)
         return list(groups)
 
     def location_data_from_ip_address(self, ip_address: str) -> dict:
@@ -176,7 +183,7 @@ class Amplitude():
         location_data['location_lng'] = lat_lon[1]
         return location_data
 
-    def device_data_from_request(self, request) -> dict:
+    def device_data_from_request(self, request: HttpRequest) -> dict:
         device_data: dict = {}
 
         user_agent = get_user_agent(request)
